<?php

namespace Drupal\syncart\Service;

use Drupal\commerce_cart\CartProviderInterface;
use Drupal\image\Entity\ImageStyle;

/**
 * Custom Cart Service.
 */
class SynRenderService implements SynRenderServiceInterface {

  /**
   * Провайдер корзины.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * Constructs a new SynCartService object.
   */
  public function __construct(CartProviderInterface $cart_provider) {
    $this->cartProvider = $cart_provider;
  }

  /**
   * Cart GET.
   */
  private function getCart($cid) {
    $cart = NULL;
    if ($cid) {
      $carts = $this->cartProvider->getCarts();
      if (count($carts)) {
        foreach ($carts as $key => $user_cart) {
          if ($user_cart->id() == $cid) {
            $cart = $user_cart;
          }
        }
      }
    }
    else {
      $cart = $this->cartProvider->getCart('default');
    }
    $this->cart = $cart;
    return $cart;
  }

  /**
   * Cart Info.
   */
  public function render($cid = FALSE, $render = []) {
    $cart = $this->getCart($cid);

    $render = [
      'price' => 0,
      'number' => 0,
      'quantity' => 0,
    ];
    if ($cart) {
      $items = $cart->getItems();
      foreach ($items as $index => $orderItem) {
        $variationInfo = $this->getVariationInfo($orderItem);
        if (empty($variationInfo)) {
          continue;
        }
        $render['id'] = $cart->id();
        $render['items'][$orderItem->id()] = $variationInfo;
        $render['quantity'] += $orderItem->getQuantity();
      }
      foreach ($cart->getAdjustments() as $index => $adjustment) {
        $adj_price = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $adjustment->getAmount(),
          ],
        ];
        $adj_price_renderable = \Drupal::service('renderer')->renderRoot($adj_price);
        $render['adjustments'][$index] = [
          'label' => $adjustment->getLabel(),
          'type' => $adjustment->getType(),
          'source' => $adjustment->getSourceId(),
          'is_included' => $adjustment->isIncluded(),
          'price' => $adj_price_renderable,
        ];
      }
      if (!empty($items)) {
        $total = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $cart->getTotalPrice(),
          ],
        ];
        $subtotal = [
          '#type' => 'inline_template',
          '#template' => '{{ price|commerce_price_format }}',
          '#context' => [
            'price' => $cart->getSubtotalPrice(),
          ],
        ];
        $total_renderable = \Drupal::service('renderer')->renderRoot($total);
        $subtotal_renderable = \Drupal::service('renderer')->renderRoot($subtotal);
        $render['price'] = $subtotal_renderable;
        $render['subtotal'] = $subtotal_renderable;
        $render['total'] = $total_renderable;
        $render['number'] = $cart->getSubtotalPrice()->getNumber();
      }
    }
    return $render;
  }

  /**
   * Получить информацию о вариации из товара в корзине.
   */
  private function getVariationInfo($orderItem, $info = []) {
    $currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    if (is_object($variation = $orderItem->getPurchasedEntity())) {
      $this->getVariationProductInfo($variation, $info);
      $currencyCode = $variation->getPrice()->getCurrencyCode();
      $stock = FALSE;
      if ($variation->field_stock) {
        $stock = intval($variation->field_stock->value);
      }
      $info['id'] = $orderItem->id();
      $info['quantity'] = round($orderItem->getQuantity());
      $info['stock'] = $stock;
      $info['number'] = $variation->getPrice()->getNumber();
      $price = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $variation->getPrice(),
        ],
      ];
      $total = [
        '#type' => 'inline_template',
        '#template' => '{{ price|commerce_price_format }}',
        '#context' => [
          'price' => $orderItem->getTotalPrice(),
        ],
      ];
      $info['price'] = \Drupal::service('renderer')->renderRoot($price);
      $info['totalPrice'] = \Drupal::service('renderer')->renderRoot($total);
      \Drupal::moduleHandler()->alter('syncart_variation', $info, $variation);
    }
    return $info;
  }

  /**
   * Получить информацию о товаре вариации.
   */
  private function getVariationProductInfo($variation, &$info) {
    if (is_object($product = $variation->getProduct())) {
      if ($product->field_title && $product->field_title->value) {
        $title = $product->field_title->value;
      }
      if ($variation->field_variation_image) {
        $variationImage = $variation->field_variation_image->entity;
        if (is_object($variationImage)) {
          $uri = $variationImage->getFileUri();
          $image = ImageStyle::load('product_cart')->buildUrl($uri);
        }
      }
      $info['title'] = $title ?? $product->getTitle();
      $info['picture'] = $image ?? $this->getProductPicture($product);
      $info['url'] = $product->toUrl()->toString();
      $info['pid'] = $product->id();

    }
  }

  /**
   * Загрузка изображения товара.
   */
  private function getProductPicture($product) {
    $image = '';
    $imageUri = '';
    $fieldImage = $product->field_image;
    $fieldGallery = $product->field_gallery;
    if ($fieldImage && !$fieldImage->isEmpty()) {
      foreach ($fieldImage as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    elseif ($fieldGallery && !$fieldGallery->isEmpty()) {
      foreach ($fieldGallery as $item) {
        $imageUri = $item->entity->getFileUri();
        break;
      }
    }
    if ($imageUri) {
      $image = ImageStyle::load('product_cart')->buildUrl($imageUri);
    }
    return $image;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $items = [];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
    }
    return empty($items);
  }

}
